/*
 * $Id: scdb.h,v 1.1.1.1 2005/05/11 09:03:12 gleissner Exp $
 *
 *
 * Project     :  SCPM (System Configuration Profile Management)
 * Module      :  SCPM database
 * File        :  scdb.hh
 * Description :  defines SCDB access and management functions
 * Author      :  Joachim Gleissner <jg@suse.de>
 *
 * Copyright 2002-2004 SuSE Linux AG
 *
 * Released under the terms of the GNU General Public License
 * (see file COPYRIGHT in project root directory).
 *
 */
#ifndef SCDB_H
#define SCDB_H

#include <vector>
#include <string>
#include <iostream>
#include <sys/stat.h>

class SCDB
{
public:
  friend class SCPM;
  friend class SCDBUtil;


  struct dbinfo_t {
    std::string version;
    std::string type;
    bool is_local;
  };

  struct resource_dep_t {
    std::string name;
    std::string type;
  };

  struct db_action_failed
  {
  public:
    db_action_failed( std::string r ) { reason = r; };
    std::string reason;
  };

  static const int db_state_ok                  = 0;
  static const int db_state_open_failed         = 1;
  static const int db_state_corrupt             = 2;
  static const int db_state_version_unsupported = 3;

  static SCDB * GetHandle( ) { return scdb_handle; };

  // scdb specific functions
  struct dbinfo_t GetDBInfo( );
  int LoadDB( )  throw( db_action_failed );
  int LoadDB( std::string dbfile )  throw( db_action_failed );
  int LoadDB( std::istream &input, bool quick=false )  throw( db_action_failed );
  void SaveDB( )  throw( db_action_failed );
  void SaveDB( std::ostream &output )  throw( db_action_failed );
  void DropDB( );
  bool Modified( ) { return modified; };
  std::string GetSCDBVersion( );
  void   SetSCDBVersion( std::string version );
  std::string GetFormat( ) { return "xml"; };
  std::string GetFormatVersion( );
  bool UpToDate();

  // scdb global functions
  void SetStatusFlag( std::string status_flag, bool value ) throw( db_action_failed );
  bool GetStatusFlag( std::string status_flag ) throw( db_action_failed );
  void SetStatusKey( std::string status_key, std::string value ) throw( db_action_failed );
  std::string GetStatusKey( std::string status_key ) throw( db_action_failed );
  void SetActiveProfile( std::string profile_name ) throw( db_action_failed );
  std::string GetActiveProfile( ) throw( db_action_failed );
  std::vector<std::string> GetProfiles( ) throw( db_action_failed );

  // resource functions
  bool ResourceExists( std::string resource_name, std::string resource_type )
    throw( db_action_failed );
  void ResourceAdd( std::string resource_name, std::string resource_type )
    throw( db_action_failed );
  void ResourceAddType( std::string resource_type )
    throw( db_action_failed );
  void ResourceDelete( std::string resource_name, std::string resource_type )
    throw( db_action_failed );
  void ResourceAddDep( std::string resource_name, std::string resource_type, std::string dep_name, std::string dep_type )
    throw( db_action_failed );
  void ResourceCleanDeps( std::string resource_name, std::string resource_type )
    throw( db_action_failed );
  std::vector<resource_dep_t> ResourceGetDeps( std::string resource_name, std::string resource_type )
    throw( db_action_failed );
  std::vector<std::string> ResourceGetTypes( )
    throw( db_action_failed );
  std::vector<std::string> ResourceGetNames( std::string resource_type )
    throw( db_action_failed );
  void ResourceAddData( std::string resource_name, std::string resource_type,
			std::string data_entry, std::string data_value )
    throw( db_action_failed );
  void ResourceClearData( std::string resource_name, std::string resource_type )
    throw( db_action_failed );
  void ResourceGetData( std::string resource_name, std::string resource_type,
			std::vector<std::string> &data_entries, std::vector<std::string> &data_values )
    throw( db_action_failed );

  // profile functions
  bool ProfileExists( std::string profile_name )
    throw( db_action_failed );
  void ProfileAdd( std::string profile_name )
    throw( db_action_failed );
  void ProfileDelete( std::string profile_name )
    throw( db_action_failed );
  void ProfileAddResource( std::string profile_name, std::string resource_name, std::string resource_type )
    throw( db_action_failed );
  void ProfileDeleteResource( std::string profile_name, std::string resource_name, std::string resource_type )
    throw( db_action_failed );
  void ProfileClearResources( std::string profile_name, std::string resource_type="all" )
    throw( db_action_failed );
  bool ProfileHasResource( std::string profile_name, std::string resource_name, std::string resource_type )
    throw( db_action_failed );
  bool ProfileGetFlag( std::string profile, std::string flag )
    throw( db_action_failed );
  void ProfileSetFlag( std::string profile, std::string flag, bool value )
    throw( db_action_failed );
  std::string ProfileGetKey( std::string profile, std::string key )
    throw( db_action_failed );
  void ProfileSetKey( std::string profile, std::string key, std::string value )
    throw( db_action_failed );
  std::string ProfileGetScript( std::string profile, std::string script_name )
    throw( db_action_failed );
  void ProfileSetScript( std::string profile, std::string script_name, std::string script_location )
    throw( db_action_failed );
  std::vector<std::string> ProfileGetResourceTypes( std::string profile )
    throw( db_action_failed );
  std::vector<std::string> ProfileGetResources( std::string profile, std::string resource_type )
    throw( db_action_failed );

  // generic access functions
  void Dump( std::ostream &output, std::string key="root" )
    throw( db_action_failed );
  void SetValue( std::string key, std::string value )
    throw( db_action_failed );
  std::string GetValue( std::string key )
    throw( db_action_failed );
  void AddNodes( std::string key, std::string value="" ) throw( db_action_failed );
  void DeleteNodes( std::string key ) throw( db_action_failed );
  void RenameNodes( std::string key, std::string name ) throw( db_action_failed );

  // SCDB file interface helpers
  struct no_such_file{};
  struct file_exists{};

  bool FileExists( const std::string &profile, const std::string &resource_type,
		   const std::string &resource_name )
    throw( db_action_failed );
  void FileSave( const std::string &profile, const std::string &resource_type,
		 const std::string &resource_name, std::istream &in, struct stat *st=NULL)
    throw( db_action_failed );
  void FileRestore( const std::string &profile, const std::string &resource_type,
		    const std::string &resource_name, std::ostream &out,
		    struct stat *st=NULL )
    throw( db_action_failed, no_such_file );
  void FileCopy( const std::string &profile, const std::string &dest_profile, const std::string &resource_type,
		 const std::string &resource_name )
    throw( db_action_failed, no_such_file, file_exists );
  void FileDelete( const std::string &profile, const std::string &resource_type,
		   const std::string &resource_name )
    throw( db_action_failed, no_such_file );
  std::string &FileGetMD5( const std::string &profile, const std::string &resource_type,
		      const std::string &resource_name )
    throw( db_action_failed, no_such_file );
  bool FileHasBackup( const std::string &profile, const std::string &resource_type,
		      const std::string &resource_name );

private:
  static SCDB * scdb_handle;
  bool modified;

};

#endif
